#!/usr/bin/env python
#encoding=utf-8

import serial
import sys
import time

try:
	import RPi.GPIO as GPIO
	IamRaspi = True
except ImportError:
	IamRaspi = False

from constant import *
from tweet import *

argv = sys.argv
argc = len(argv)

#	グローバル変数
tstart = time.time()
ctime = [ tstart, tstart ]
state = [ -1, -1 ]
nowplace = [ -1, -1 ]
conNum = [ -1, -1 ]
tstamp = [ "", "" ]
raddr = [ "", "" ]
LQI = [ "", "" ]
power = [ "", "" ]
AI1 = [ "", "" ]
AI3 = [ "", "" ]
accelX = [ "", "" ]
accelY = [ "", "" ]
accelZ = [ "", "" ]

#	GPIOを初期化
def initGPIO():
	#	GPIO番号を用いる
	GPIO.setmode(GPIO.BCM)
	i = 0
	j = 0

	#	GPIOを出力に設定してLo(消灯)する
	while i < 2:
		while j < 7:
			try:
				GPIO.setup( catGPIO[i][j], GPIO.OUT )
				GPIO.output( catGPIO[i][j], False )
			except NameError:
				return

			j += 1

		i += 1
		j = 0

#	LEDを点灯させる
def chGPIO( catNum, state ):
	i = 0
	#	指定した猫のLEDを一旦すべて消す
	while i < 7:
		try:
			GPIO.output( catGPIO[catNum][i], False )
		except NameError:
			return

		i += 1

	#	指定したポートのみ点灯
	try:
		GPIO.output( catGPIO[catNum][state], True )
	except NameError:
		return

#	状態の変更と表示など
def chState( catNum, nstate ):
	#	状態の更新
	state[catNum] = nstate
	#	表示
	#print catName[catNum] + ":" + tweetStr[state[catNum]]
	#	LEDの更新
	if IamRaspi == True:
		chGPIO( catNum, state[catNum] )
	#	ツイート
	post_tweet( catNum, state[catNum] )

#	加速度などの値の更新(ひとつ前のだけ保持)
def valueRefresh( catNum, *sline ):
	#	値の更新
	tstamp[catNum] = sline[1]
	raddr[catNum] = sline[2]
	LQI[catNum] = sline[3]
	conNum[catNum] = int(sline[4])
	power[catNum] = sline[6]
	AI1[catNum] = sline[9]
	AI3[catNum] = sline[10]
	accelX[catNum] = sline[12]
	accelY[catNum] = sline[13]
	accelZ[catNum] = sline[14]

#	UARTの出力を読み込んで様々な処理を行う(実質メイン関数)
def readSerial( ser ):
	while True:
		#	UARTを読み込んでリスト化(配列に代入)
		line = ser.readline()
		field = line.strip()
		sline = field.split(';')
		length = len(sline)

		#	猫(が使っているセンサ)から来たデータの場合処理を行う
		if length == 16 and  sline[11] == "X":
			#	猫から来たものかどうかの判別
			try:
				catNum = catID.index(sline[5])
				test = int(sline[12])			#	数字が入っているかどうかのテスト
				test = int(sline[13])
			except ValueError:
				continue

			#	加速度が閾値以上なら走っている
			if int(sline[12]) > thAccel or int(sline[13]) > thAccel:
				#	走っていない状態のときからここ来たときに更新する
				if state[catNum] !=  5 :
					#	状態の更新
					chState( catNum, 5 )

				#	時間と各パラメータの更新
				ctime[catNum] = time.time()
				valueRefresh( catNum, *sline )

			#	初期状態でない場合
			elif conNum[catNum] != -1 :
				#	続き番号が前と同じ場合
				if conNum[catNum] == int(sline[4]):
					#	LQIが前来たのより大きい場合、LQIや中継器アドレスを更新
					if int(LQI[catNum]) < int(sline[3]) :
						tstamp[catNum] = sline[1]
						raddr[catNum] = sline[2]
						LQI[catNum] = sline[3]
				else:
					#	現在の状態の判定
					try:
						plcNum = placeID.index(raddr[catNum])
					except ValueError:
						valueRefresh( catNum, *sline )
						continue

					#	現在時刻を取得
					ntime = time.time()

					#	寝ているかどうかの判定
					if ntime-ctime[catNum] > sleep_time :
						#	前回寝ていなければ寝た状態に更新
						if state[catNum] != 1 and state[catNum] != 0:
							#	状態の更新
							chState( catNum, 0 )
						#	前にいた場所と同じ場所の場合
						if nowplace[catNum] == plcNum :
							#	値を更新してもう一度読み込み
							valueRefresh( catNum, *sline )
							continue

					#	LQIが閾値より大きい場合
					if int(LQI[catNum]) > thLQI:
						#	前の状態と異なる場合更新
						if state[catNum] !=  plcNum :
							#	時間の更新
							ctime[catNum] = time.time()
							#	状態の更新
							chState( catNum, plcNum )
							#	現在いる場所の更新
							nowplace[catNum] = plcNum

					#	各種パラメータの更新
					valueRefresh( catNum, *sline )

			#	初回かつ加速度が出ていない場合
			else:
				#	各種パラメータの更新
				valueRefresh( catNum, *sline )

if __name__ == "__main__":
	if IamRaspi == True:
		initGPIO()

	#	引数がある場合、引数で指定したデバイスを用いる
	if argc != 2:
		ser = serial.Serial( "/dev/ttyUSB0", 115200, timeout=0 );
	else :
		ser = serial.Serial( argv[1], 115200, timeout=0 )

	#	シリアルデータを読み込み関数を別スレッドで実行
	while True:
		try:
			readSerial( ser )
		except KeyboardInterrupt:
			break
		except:
			continue

	GPIO.cleanup()
	ser.close()
